#include "api.h"


Window::Window()
{ }

Window::~Window()
{ }

LRESULT
Window::handle(HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam)
{
    return DefWindowProc(hwnd, msg, wparam, lparam);
}

LRESULT CALLBACK
Window::WindowProc(HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam)
{
    if (msg == WM_CREATE) {
        // We could remember our pointer here
        LPCREATESTRUCT ps = (LPCREATESTRUCT) lparam;
        SetWindowLong(hwnd, GWL_USERDATA, LONG(ps->lpCreateParams));
        return 0;
    } else if (msg == WM_NCCREATE || msg == WM_NCDESTROY) {
        // Do not handle these, our pointer may already be dead
        return DefWindowProc(hwnd, msg, wparam, lparam);
    } else {
        Window* me = reinterpret_cast<Window*>(GetWindowLong(hwnd, GWL_USERDATA));
        if (me) {
            return me->handle(hwnd, msg, wparam, lparam);
        } else {
            return DefWindowProc(hwnd, msg, wparam, lparam);
        }
    }
}

void
Window::registerWindowClass(const char* name, HICON icon)
{
    WNDCLASS wc;
    wc.style         = CS_HREDRAW | CS_VREDRAW;
    wc.lpfnWndProc   = Window::WindowProc;
    wc.cbClsExtra    = 0;
    wc.cbWndExtra    = 0;
    wc.hInstance     = GetModuleHandle(0);
    wc.hIcon         = icon;
    wc.hCursor       = LoadCursor(0, IDC_ARROW);
    wc.hbrBackground = (HBRUSH) GetStockObject(WHITE_BRUSH);
    wc.lpszMenuName  = 0;
    wc.lpszClassName = name;
    RegisterClass(&wc);
}

HWND
Window::create(LPCTSTR lpClassName,         // pointer to registered class name
               LPCTSTR lpWindowName,        // pointer to window name
               DWORD dwStyle,               // window style
               int x,                       // horizontal position of window
               int y,                       // vertical position of window
               int nWidth,                  // window width
               int nHeight,                 // window height
               HWND hWndParent,             // handle to parent or owner window
               HMENU hMenu)                // handle to menu or child-window identifier
{
    return CreateWindow(lpClassName, lpWindowName, dwStyle,
                        x, y, nWidth, nHeight,
                        hWndParent, hMenu,
                        GetModuleHandle(0), this);
}
